//
// Cthugha - Audio Seeded Image Processing
//
// Zaph, Digital Aasvogel Group, Torps Productions 1993-1994
//


#include <stdio.h>
#include <dos.h>
#include <io.h>
#include <fcntl.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>
#include <bios.h>
#include <memory.h>
#include <assert.h>

#include "cthugha.h"
#include "charset.h"
#include "zorilkey.h"
#include "options.h"

int olddisp=0,xoffs=0,yoffs=0,dxoffs=0,dyoffs=0;
void display_up(void);
void display_dn(void);

void (*display)(void);

char norefresh=TRUE;

void wait_vsync(void)
{

	_asm {
	    mov     dx,0x03da
WaitVS:
	    in      al,dx
    	test    al,0x08
	    jz      WaitVS  ;vertical sync is active high (1 = active)
	}

}

void display_up(void);
void display_dn(void);
void display_2hor(void);
void display_r2hor(void);
void display_4hor(void);
void display_2verd(void);
void display_r2verd(void);
void display_4kal(void);

int numdisplays=-1;

function_opt disparray[]={
	{ display_up,		WHEN_ALWAYS, "Upwards" },
	{ display_dn,		WHEN_ALWAYS, "Downwards" },
	{ display_2hor,		WHEN_ALWAYS, "Hor. Split out" },
	{ display_r2hor,	WHEN_ALWAYS, "Hor. Split in" },
	{ display_4hor,		WHEN_ALWAYS, "Kaleidescope" },
	{ display_2verd,	WHEN_ALWAYS, "90deg rot. mirror" },
	{ display_r2verd,	WHEN_ALWAYS, "90deg rot. mirror 2" },
	{ display_4kal,		WHEN_ALWAYS, "90deg Kaleidescope" },
	{ NULL,				WHEN_NEVER, "<BAD>"}
};

int change_display(int disp)
{
	if (numdisplays<0) {
		numdisplays=0;
		while (disparray[numdisplays].function!=NULL)
			numdisplays++;
		assert(numdisplays);
	}

	disp=disp%numdisplays;

	disp=check_disp_ok(disp);

	display=disparray[disp].function;

	return disp;

}
void display_up(void)
{
	char *screen;

	FP_SEG(screen)=0xa000;
	FP_OFF(screen)=0;

	memcpy(screen,buff,64000);
}


void display_dn(void)
{
	char *screen;
	int i;

	FP_SEG(screen)=0xa000;
	FP_OFF(screen)=0;

	for (i=199; i>=0; i--) {
		memcpy(screen,buff[i],320);
		screen+=320;
	}
}

void display_2hor(void)
{
	char *screen,*puff;
	int x,y;
	int use_temp=0;

	FP_SEG(screen)=0xa000;
	FP_OFF(screen)=0;
	puff=(char*)buff;

	for(y=99;y>=0;y--){
		memcpy(screen,buff[y+100],320);
		screen+=320;
	}

	memcpy(screen,buff[100],(32000));
}

void display_r2hor(void)
{
	char *screen,*puff;
	int x,y;
	int use_temp=0;

	FP_SEG(screen)=0xa000;
	FP_OFF(screen)=0;
	puff=(char*)buff;

	memcpy(screen,buff[100],(32000));
	screen += (32000);

	for(y=99;y>=0;y--){
		memcpy(screen,buff[y+100],320);
		screen+=320;
	}
}

void display_4hor(void)
{
	char *screen,*puff;
	int x,y;
	int use_temp=0;

	FP_SEG(screen)=0xa000;
	FP_OFF(screen)=0;
	puff=(char*)buff;

	for(y=0;y<=99;y++){
		memcpy(screen,buff[y+100],160);
		for(x=0;x<160;x++)
			*(screen+160+x)=*(buff[y+100]+159-x);
		screen+=320;
	}
	for(y=99;y>=0;y--){
		memcpy(screen,buff[y+100],160);
		for(x=0;x<160;x++)
			*(screen+160+x)=*(buff[y+100]+159-x);
		screen+=320;
	}

}

void display_2verd(void)
{
	char *screen,*puff;
	int x,y;
	int use_temp=0;

	FP_SEG(screen)=0xa000;
	FP_OFF(screen)=0;
	puff=(char*)buff;

	for(y=0;y<200;y++){
		for(x=0;x<160;x++){
			*screen++=*puff;    //buff[x][y];
			puff+=320;
		}
		for( ;x<320;x++){
			*screen++=*puff;   //buff[320-x][y];
			puff-=320;
		}
		puff++;
	}

}
void display_r2verd(void)
{
	char *screen,*puff;
	int x,y;
	int use_temp=0;

	FP_SEG(screen)=0xa000;
	FP_OFF(screen)=0;
	puff=(char*)buff;


	screen+=64000u;
	for(y=0;y<200;y++){
		for(x=0;x<160;x++){
			*screen--=*puff;    //buff[x][y];
			puff+=320;
		}
		for( ;x<320;x++){
			*screen--=*puff;   //buff[320-x][y];
			puff-=320;
		}
		puff++;
	}

}
void display_4kal(void)
{
	char *screen,*puff;
	int x,y;
	int use_temp=0;

	FP_SEG(screen)=0xa000;
	FP_OFF(screen)=0;
	puff=(char*)buff;

	for(y=0;y<100;y++){
		for(x=0;x<160;x++){
			*screen++=*puff;    //buff[x][y];
			puff+=320;
		}
		for( ;x<320;x++){
			*screen++=*puff;   //buff[320-x][y];
			puff-=320;
		}
		puff++;
	}
	puff=(char*) buff;
	screen+=32000u-1;
	for(y=0;y<100;y++){
		for(x=0;x<160;x++){
			*screen--=*puff;    //buff[x][y];
			puff+=320;
		}
		for( ;x<320;x++){
			*screen--=*puff;   //buff[320-x][y];
			puff-=320;
		}
		puff++;
	}
}

#if 0
void display(void)
{
        char *screen,*puff;
        int x,y;
		int use_temp=0;

        FP_SEG(screen)=0xa000;
        FP_OFF(screen)=0;
        puff=(char*)buff;

		if (!norefresh)
			wait_vsync();

        switch(curdisplay){
          case 0:
          case 8:
          default:
			display_up();
            break;

          case 1:
          case 9:
			display_dn();
			break;
          case 2:                       //cal2 hor
			display_2hor();
            break;
          case 3:                       //cal2 hor
			display_r2hor();
                break;
          case 4:               // cal4 hor
			display_4hor();
                break;

         case 5:                //2cal ver div
			display_2verd();
                break;          
         case 6:                //2cal ver div
			display_r2verd();
                break;          
         case 7:                        //4-cal
			display_4kal();
                break;          
        }
        //olddisp=curdisplay;

}

const int NUMDISPLAYS=10;
#endif


void draw_text(int xpos, int ypos, int size, int colour, char *tbuf)
{
	int    x,y,i,j;
	unsigned char                   mask;
	unsigned char *cdef_ptr;

	if (size<=0)
   		return;

	while (*tbuf) {
	   	cdef_ptr = charset[*tbuf];
   		for (y=ypos; y<=ypos+(size*char_height)-1; y+=size) {
			mask=0x80;
			for (x=xpos; x<xpos+(size*(char_width+1)); x+=size) {
				if ((*cdef_ptr)&mask) {
					if (size==1)
						if (curdisplay)
							buff[199-y][x]=colour;
						else
							buff[y][x]=colour;
					else
						for (i=0; i<size; i++)
							for (j=0; j<size; j++)
								if (curdisplay)
									buff[199-(y+i)][(x+j)]=colour;
								else
									buff[y+i][x+j]=colour;
				}
				mask>>=1;
				}
			cdef_ptr++;
		}
   		tbuf++;
		xpos+=size*(char_width+1);
  	}
}

